#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === GenerateRcFile
#
# This class will generate a single standalone rc file.
# =========================================================================== #
require 'opn'
begin
  require 'colours'
rescue LoadError; end
require 'yaml'
require 'fileutils'
require 'generate_rc_file/version/version.rb'
require 'generate_rc_file/constants.rb'

class GenerateRcFile # require 'x/tools/generate_rc_file.rb'; GenerateRcFile.new

  include Colours

  attr_accessor :use_putty_style_aliases

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline(i)
    parse_commandline
    run if run_already
  end

  # ========================================================================= #
  # === base_directory?
  # ========================================================================= #
  def base_directory?
    @base_dir
  end; alias base_dir? base_directory? # === base_dir?

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    set_base_dir(BASE_DIR)
    @use_putty_style_aliases = false # if true we use putty-style aliases.
  end

  # ========================================================================= #
  # === enable_putty_style
  # ========================================================================= #
  def enable_putty_style
    @use_putty_style_aliases = true
  end

  # ========================================================================= #
  # === ensure_main_encoding
  #
  # Ensure a uniform encoding here.
  # ========================================================================= #
  def ensure_main_encoding(
      i,
      use_this_encoding = ISO_ENCODING
    )
    if i.is_a?(String)
      i = i.dup if i.frozen?
      i = i.force_encoding(use_this_encoding)
    end
    i
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'You can use another target directory.'
    e
    e 'To do this, do:'
    e
    e '  generate_rc_file --dir=~/AUTOGENERATED'
    e
  end

  # ========================================================================= #
  # === parse_commandline
  # ========================================================================= #
  def parse_commandline(i = @commandline)
    @commandline.each {|entry|
      case entry # case tag
      # ===================================================================== #
      # === gcfile --dir=~/AUTOGENERATED
      # ===================================================================== #
      when /-?-?dir=(.+)/ # See: http://rubular.com/r/kF3uKcOoL9
        set_base_dir($1.to_s.dup)
        try_to_create_the_base_directory_if_it_does_not_exist(base_dir?) # Also ensure that the directory exists.
      # ===================================================================== #
      # === --help
      # ===================================================================== #
      when /-?-?help/i
        show_help
        exit
      end
    }
    purge_entries_with_two_hyphens
  end; alias menu parse_commandline # === menu

  # ========================================================================= #
  # === purge_entries_with_two_hyphens
  # ========================================================================= #
  def purge_entries_with_two_hyphens
    @commandline.reject! {|entry| entry.to_s.start_with? '--' }
  end

  # ========================================================================= #
  # === set_commandline
  # ========================================================================= #
  def set_commandline(i)
    i = [i] unless i.is_a? Array
    @commandline = i
  end

  # ========================================================================= #
  # === set_store_here
  #
  # The argument to this method shall always be the absolute path.
  # ========================================================================= #
  def set_store_here(i)
    @store_here = i
  end

  # ========================================================================= #
  # === add_header_for
  #
  # Use this method if you want to return the default header.
  # ========================================================================= #
  def add_header_for(this_file) # Add the default header here.
    HEADER1+this_file+'_rc'+N+HEADER2
  end

  # ========================================================================= #
  # === load_yaml
  # ========================================================================= #
  def load_yaml(i) # A simple, failsafe yaml-wrapper.
    begin
      YAML.load_file(i)
    rescue Exception => error
      opn; e 'We encountered an error in GenerateRCFile, in '\
             'the method load_yaml():'
      pp error
    end
  end

  # ========================================================================= #
  # === do_generate_the_rc_file
  # ========================================================================= #
  def do_generate_the_rc_file(use_this_input = @commandline)
    if use_this_input.is_a? Array
      if use_this_input.empty?
        use_this_input << ARRAY_ALL_FILES # Add all possible rc files here.
      else
        case use_this_input
        when 'all',:all
          use_this_input << ARRAY_ALL_FILES
        when 'cd',:cd
          use_this_input << 'cd_aliases'
        end
      end
    end
    use_this_input.flatten.each {|generate_this_file|
      generate_this_file = generate_this_file.to_s # We need a String.
      full_path_to_the_file = LINUX_YAML+generate_this_file+'.yml'
      # ===================================================================== #
      # Next, add some exceptions:
      # ===================================================================== #
      if    full_path_to_the_file.include? 'aliases.yml'
        full_path_to_the_file = RCFILE_GENERATOR_YAML_DIRECTORY+'aliases.yml'
      elsif full_path_to_the_file.include? 'cd_aliases.yml'
        full_path_to_the_file = RCFILE_GENERATOR_YAML_DIRECTORY+'cd_aliases.yml'
      elsif full_path_to_the_file.include? 'programs_aliases.yml'
        full_path_to_the_file = RCFILE_GENERATOR_YAML_DIRECTORY+'programs_aliases.yml'
      end
      hash_dataset = load_yaml(full_path_to_the_file)
      _ = ''.dup
      _ << add_header_for(generate_this_file) # Add the default header here.
      # ===================================================================== #
      # We have to find out whether we will use alias-style or export-style
      # next. For alias-style, we also may have to use putty-style or
      # normal-style.
      # ===================================================================== #
      if ARRAY_WHICH_FILES_TO_ALIAS.include? generate_this_file
        # =================================================================== #
        # Ok, alias-style comes here.
        # =================================================================== #
        hash_dataset.each_pair {|key, value|
          key   = ensure_main_encoding(key.to_s)
          value = ensure_main_encoding(value.to_s).strip
          if @use_putty_style_aliases
            _ << "alias "+key.to_s+'="'+value+'"'+N
          else
            if value.include? "'" # Need to use another syntax for included ' characters.
              _ << "alias '#{key}'=\"#{value}\"#{N}"
            else
              _ << "alias '#{key}'='#{value}'#{N}"
            end
          end
        }
      else
        # =================================================================== #
        # Ok, in this case we assume export-style.
        # =================================================================== #
        hash_dataset.each_pair {|key, value|
          value = value.to_s
          _ << "export "+key.to_s+'="'+value+'"'+N
        }
      end
      set_store_here(@base_dir+generate_this_file+'_rc')
      store_what_where(_, @store_here)
    }
  end

  # ========================================================================= #
  # === set_base_dir
  # ========================================================================= #
  def set_base_dir(i = BASE_DIR)
    i = i.to_s.dup # Work on a "fresh" copy.
    # ======================================================================= #
    # Must check if the input contains a ~ character, which is then assumed
    # to refer to the home dir of the user.
    # ======================================================================= #
    if i.include? '~'
      i = File.expand_path(i)
    end
    i << '/' unless i.end_with? '/'
    @base_dir = i
  end

  # ========================================================================= #
  # === try_to_create_the_base_directory_if_it_does_not_exist
  # ========================================================================= #
  def try_to_create_the_base_directory_if_it_does_not_exist(i = @base_dir)
    unless File.directory? i
      FileUtils.mkdir_p(i) if File.writable?(File.dirname(i))
    end
  end

  # ========================================================================= #
  # === store_what_where
  # ========================================================================= #
  def store_what_where(
      what  = @_,
      where = base_directory?+'aliases_rc'
    ) # store what, where.
    opn; e 'Now storing at `'+sfile(where)+'`.'
    File.open(where, 'w+') {|f| f.puts what }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    do_generate_the_rc_file
  end

end

if __FILE__ == $PROGRAM_NAME 
  GenerateRcFile.new(ARGV)
end # generate_rc_file --help
    # generate_rc_file all
    # generate_rc_file cd_aliases